__version__ = "0.6"
__license__ = "GNU GPL"

import gtk,gtk.glade,gobject,gc,os,threading,time
from time import gmtime, strftime

class ZenShot:
	def __init__(self):
		self.xml = gtk.glade.XML('/usr/share/zenshot/gui.glade', 'window')
		self.window = self.xml.get_widget('window')
		self.window.set_icon_from_file('/usr/share/pixmaps/zenshot.png')

		self.about = self.xml.get_widget('about')
		self.about.set_image(gtk.image_new_from_stock(gtk.STOCK_ABOUT,gtk.ICON_SIZE_SMALL_TOOLBAR))

		self.change_dir = self.xml.get_widget('change_dir')
		self.change_dir.set_image(gtk.image_new_from_stock(gtk.STOCK_DIRECTORY,gtk.ICON_SIZE_SMALL_TOOLBAR))

		self.filename = self.xml.get_widget('filename')
		self.combo = self.xml.get_widget('format')
		f_list = gtk.ListStore(str)
		f_list.append(["png"])
		f_list.append(["jpg"])
		cell = gtk.CellRendererText()
		self.combo.pack_start(cell)
		self.combo.add_attribute(cell, 'text', 0)
		self.combo.set_model(f_list)
		self.combo.set_active(0)
		self.combo.connect('changed', self.format_change)

		self.save_path=''
		self.save_format = 'png'
		self.timer_id = None
		self.shot = None

		self.shot_button = self.xml.get_widget('button2')
		self.image = self.xml.get_widget('image')
		self.screen_shot()
		self.quality = self.xml.get_widget('quality')
		self.quality.set_sensitive(False)
		self.delay = self.xml.get_widget('delay')
		self.delay.set_sensitive(True)

		self.xml.signal_autoconnect(self)
		self.window.show_all()

	def select_dir_button_clicked(self, widget):
		dialog = gtk.FileChooserDialog(("Choose a directory"),self.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,(("Cancel"), gtk.RESPONSE_CANCEL, ("Choose"), gtk.RESPONSE_OK))
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.save_path = dialog.get_filename()
		dialog.destroy()

	def format_change(self, widget):
		model = widget.get_model()
		index = widget.get_active()
		if index > -1:
			if model[index][0]=="png":
				self.quality.set_sensitive(False)
				self.save_format = 'png'
			if model[index][0]=="jpg":
				self.quality.set_sensitive(True)
				self.save_format = 'jpg'
		return

	def shot_button_clicked(self,widget):
		self.check_timer()

	def check_timer(self):
		if self.delay.get_value_as_int() == 0:
			if self.timer_id: gobject.source_remove(self.timer_id)
			self.screen_shot()
			self.shot_button.set_sensitive(True)
		else:
			self.shot_button.set_sensitive(False)
			self.delay.set_value(int(self.delay.get_value_as_int()-1))
			self.timer_id = gobject.timeout_add(1000, self.check_timer)

	def screen_shot(self):
		self.window.hide()

		gtk.gdk.flush()
		time.sleep(0.4)
		gtk.gdk.flush()

		if self.shot > 0: del self.shot
		gc.collect()

		width=400
		height=250
		s_width = gtk.gdk.screen_width()
		s_height = gtk.gdk.screen_height()

		w = round(s_width * height / s_height);
		h = round(s_height * width / s_width);
		if (height-h) < (width-w): width = int(w);
		else: height = int(h)

		self.shot = gtk.gdk.Pixbuf.get_from_drawable(
             gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, s_width , s_height),
             gtk.gdk.get_default_root_window(),
             gtk.gdk.colormap_get_system(),
             0, 0, 0, 0, s_width, s_height)

		scaled_buf = self.shot.scale_simple(width,height,gtk.gdk.INTERP_BILINEAR)
		self.image.set_from_pixbuf(scaled_buf)
		self.window.show()

	def save_button_clicked(self,widget):
		if self.save_path != '': path = self.save_path
		elif os.path.isdir(os.path.expanduser('~')+'/Desktop'): path = os.path.expanduser('~')+'/Desktop'
		else: path = os.path.expanduser('~')
		savefile = self.filename.get_text()+" "+strftime("%d %b %Y %H:%M:%S", time.localtime(time.time()))+"."+self.save_format
		if self.save_format=="png": self.shot.save(path+"/"+savefile, "png")
		else: self.shot.save(path+"/"+savefile, "jpeg", {"quality":str(self.quality.get_value_as_int())})
		self.window.set_title(savefile)

	def about_button_clicked(self,widget):
		dialog = gtk.AboutDialog()
		dialog.set_name('ZenShot')
		dialog.set_version(__version__)
		dialog.set_comments('Simple screenshot tool')
		dialog.set_website('xrado.hopto.org')
		dialog.run()
		dialog.destroy()

	def on_window_delete_event(self, widget, event):
		gc.collect()
		gtk.main_quit()

	def main(self):
		gtk.main()
